import com.sun.opengl.util.GLUT;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureCoords;
import com.sun.opengl.util.texture.TextureIO;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.media.opengl.*;
import javax.media.opengl.glu.GLU;
import java.io.*;

/**
* For our purposes only two of the
* GLEventListeners matter. Those would
* be init() and display().
*/
public class RoomView implements GLEventListener, KeyListener
{
    double xPosition = 0;
    double zPosition = 4;
    double yPosition = 4;
    int angley = 0;
    int anglex = 0;
    int chair;
    int woodenCube;

    GLCanvas glc;
    private Texture wallTexture;
    private Texture floorTexture;
    private Texture ceilingTexture;
    private Texture woodTexture;
    private Texture paint1Texture;
    private Texture paint2Texture;
    private Texture paint3Texture;
    private Texture paint4Texture;
    private Texture paint5Texture;

    float[] materialGreen = {0.2f, 1.0f, 0.7f, 1.0f};
    float[] materialWhite = {1.0f, 1.0f, 1.0f, 1.0f};
    float[] materialBlack = {0.3f,0.3f,0.0f,0.0f};


     void setGLCanvas(GLCanvas glcanvas) {
        this.glc = glcanvas;
    }
/**
* Take care of initialization here.
*/

 public void init(GLAutoDrawable gld) {

     
     wallTexture = load("wall.jpg");
     floorTexture = load("floor.jpg");
     ceilingTexture = load("ceiling.jpg");
     woodTexture = load("wood.jpg");
     paint1Texture = load("images1.jpg");
     paint2Texture = load("images2.jpg");
     paint3Texture = load("images3.jpg");
     paint4Texture = load("images4.jpg");
     paint5Texture = load("images5.jpg");

     GLUT glut = new GLUT();
     GL gl = gld.getGL();
      GLU glu = new GLU();

     WoodList(gl,glut);
     buildChairList(gl,glut);

      gl.glEnable(GL.GL_LIGHTING);
      gl.glEnable(GL.GL_LIGHT0);
      gl.glEnable(GL.GL_LIGHT1);
      

      // prepare spotlight
      float spot_ambient[] =  {0.2f,0.2f,0.2f,1.0f };
      float spot_diffuse[] =  {0.8f,0.8f,0.8f,1.0f };
      float spot_specular[] =  {0.8f,0.8f,0.8f,1.0f };

      gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT,  spot_ambient,0);
      gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE,  spot_diffuse,0);
      gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, spot_specular,0);
       
      // prepare a general fixed light
      float fix_ambient[] =  {0.1f,0.1f,0.1f,1.0f };
      float fix_diffuse[] =  {0.3f,0.3f,0.3f,1.0f };
      float fix_specular[] =  {0.1f,0.1f,0.1f,1.0f };

      gl.glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT,  fix_ambient,0);
      gl.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE,  fix_diffuse,0);
      gl.glLightfv(GL.GL_LIGHT1, GL.GL_SPECULAR, fix_specular,0);

      // smooth the drawing
      gl.glShadeModel(GL.GL_SMOOTH);

      gl.glMatrixMode(GL.GL_PROJECTION);
      gl.glLoadIdentity();

      // Aspect is width/height
      double w = ((Component) gld).getWidth();
      double h = ((Component) gld).getHeight();
      double aspect = w / h;

      //When using gluPerspective near and far need
      //to be positive.
      //The arguments are:
      //fovy, aspect, near, far
      glu.gluPerspective(60.0, aspect, 1.0, 200.0);
      glc.repaint();

    }

/**
* Take care of drawing here.
*/
public void display(GLAutoDrawable drawable) {

            GL gl = drawable.getGL();
            GLU glu = new GLU();
            GLUT glut = new GLUT();
            gl.glMatrixMode(GL.GL_MODELVIEW);
            gl.glLoadIdentity();
            gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

            //Define points for eye, at and up.
            //This is your camera. It ALWAYS goes
            //in the GL_MODELVIEW matrix.
            glu.gluLookAt(xPosition, yPosition, zPosition, xPosition, yPosition, (zPosition+20), 0, 1, 0);
            gl.glClear(GL.GL_COLOR_BUFFER_BIT);

            // set spotlight position
            float spot_position[] = {0f, 8f, 9f, 0f};
            gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION,  spot_position,0);

            //set spotlight direction
            float spot_direction[] = {0.0f,-1.0f,0.0f};
            gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPOT_DIRECTION,spot_direction,0);

            //set spotlight cut-off
            float spot_angle=90.0f;
            gl.glLightf(GL.GL_LIGHT0, GL.GL_SPOT_CUTOFF,(float)spot_angle);

            //GL_DEPTH_TEST prevents us from seeing polygons
            //that should be obscured. Remember to clear the
            //GL.GL_DEPTH_BUFFER_BIT
            gl.glEnable(GL.GL_DEPTH_TEST);
            //Notice the depth buffer bit is also being cleared
            gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

            //rotate the scene
            gl.glRotated(angley, 0, 1, 0);
            gl.glRotated(anglex, 1, 0, 0);

            //set the color and draw a teapot
            float materialpot[] = {0.2f, 0.4f, 0.6f, 0.3f};
            float shine=0.25f;
            gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialpot,0);
            gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialpot,0);
            gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,shine*128.0f);

            gl.glPushMatrix();
            gl.glTranslated(0,2.5,9);
            glut.glutSolidTeapot(0.5);
            gl.glPopMatrix();

            //draw each wall in two parts
            drawWall(gl,-10,9,-1,2,true);
            drawWall(gl,10,9,-1,2,false);

            //draw the four paintings
            drawPainting1(gl);
            drawPainting2(gl);
            drawPainting3(gl);
            drawPainting4(gl);
            drawPainting5(gl);

            //draw the floor
            drawFloorCeiling(gl,-1,true);

            //draw the ceiling
            drawFloorCeiling(gl,9,false);

            //draw the table and the four chairs,
            //first enable the tableTexture and
            //disable it when drawing finished
            woodTexture.enable();
            woodTexture.bind();
            drawTable(gl,glut);
            drawChairs(gl);
            woodTexture.disable();

            //draw the light
            drawLight(glut,gl);
            

}

/**
* Draw two of the four walls of the room,
* draw each wall as a separate part and
* place one part after the other so they
* look like one single wall
**/
void drawWall(GL gl,int x,int y1,int y2,int z,boolean flag){

      // set wall material
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialWhite,0);

      //get the image coordinates and
      //save them to new variables
      TextureCoords tc = wallTexture.getImageTexCoords ();
      float tx1 = tc.left ();
      float ty1 = tc.top ();
      float tx2 = tc.right ();
      float ty2 = tc.bottom ();

      // Enable  the two-dimensional wallTexture
      wallTexture.enable ();

      // Bind this texture to the current rendering context
      wallTexture.bind ();

      gl.glPushMatrix();

      //Draw the first wall as a total of two parts
      gl.glBegin(GL.GL_QUADS);
      for (int i=0;i<2;i++){
         gl.glVertex3i(x,y1, z);
         gl.glTexCoord2f (ty1, tx2);
         gl.glVertex3i(x,y2, z);
         gl.glTexCoord2f (ty2, tx2);
         gl.glVertex3i(x, y2, z+8);
         gl.glTexCoord2f (ty2, tx1);
         gl.glVertex3i(x, y1, z+8);
         gl.glTexCoord2f(ty1,tx1);

         z+=7;
     }

     if (flag == false){
         z=z-15;
         x=-x;
     }

     //Draw the second wall as a total of two parts
     for (int i=0;i<2;i++){
        gl.glVertex3i(x,y1, z+1);
        gl.glTexCoord2f (ty1, tx2);
        gl.glVertex3i(x,y2, z+1);
        gl.glTexCoord2f (ty2, tx2);
        gl.glVertex3i(x+10, y2, z+1);
        gl.glTexCoord2f (ty2, tx1);
        gl.glVertex3i(x+10, y1, z+1);
        gl.glTexCoord2f(ty1,tx1);

        x+=10;
     }
     gl.glEnd();

     gl.glPopMatrix();

     wallTexture.disable();
}

/**
* Draw the first painting
**/
void drawPainting1(GL gl){

     // set the material of the first painting
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialWhite,0);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialWhite,0);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialWhite,0);

     //get the image coordinates and
     //save them to new variables
     TextureCoords tc = paint1Texture.getImageTexCoords ();
     float tx1 = tc.left ();
     float ty1 = tc.top ();
     float tx2 = tc.right ();
     float ty2 = tc.bottom ();

     paint1Texture.enable ();

     paint1Texture.bind ();

     gl.glPushMatrix();

     // draw the context and apply the first painting
     gl.glBegin(GL.GL_QUADS);
     gl.glVertex3f(-6,7,16.99f);
     gl.glTexCoord2f (ty1, tx2);
     gl.glVertex3f(-6,4, 16.99f);
     gl.glTexCoord2f (ty2, tx2);
     gl.glVertex3f(-3, 4, 16.99f);
     gl.glTexCoord2f (ty2, tx1);
     gl.glVertex3f(-3, 7, 16.99f);
     gl.glTexCoord2f(ty1,tx1);
     gl.glEnd();

     gl.glPopMatrix();

     paint1Texture.disable();
}

/**
* Draw the second painting
**/
void drawPainting2(GL gl){

      // the process is the same as in the drawPainting1
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialWhite,0);

      TextureCoords tc = paint2Texture.getImageTexCoords ();
      float tx1 = tc.left ();
      float ty1 = tc.top ();
      float tx2 = tc.right ();
      float ty2 = tc.bottom ();

      paint2Texture.enable ();

      paint2Texture.bind ();

     gl.glPushMatrix();

     gl.glBegin(GL.GL_QUADS);
     gl.glVertex3f(-9.99f,6,12);
     gl.glTexCoord2f (ty1, tx2);
     gl.glVertex3f(-9.99f,3, 12);
     gl.glTexCoord2f (ty2, tx2);
     gl.glVertex3f(-9.99f, 3, 7);
     gl.glTexCoord2f (ty2, tx1);
     gl.glVertex3f(-9.99f, 6, 7);
     gl.glTexCoord2f(ty1,tx1);
     gl.glEnd();

     gl.glPopMatrix();

     paint2Texture.disable();
}

/**
* Draw the third painting
**/
void drawPainting3(GL gl){

     // the process is the same as in the drawPainting1
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialWhite,0);

      TextureCoords tc = paint3Texture.getImageTexCoords ();
      float tx1 = tc.left ();
      float ty1 = tc.top ();
      float tx2 = tc.right ();
      float ty2 = tc.bottom ();
      
      paint3Texture.enable ();
      
      paint3Texture.bind ();

      gl.glPushMatrix();
      gl.glBegin(GL.GL_QUADS);
      gl.glVertex3f(-2,6,2.01f);
      gl.glTexCoord2f (ty1, tx2);
      gl.glVertex3f(-2,3, 2.01f);
      gl.glTexCoord2f (ty2, tx2);
      gl.glVertex3f(2, 3, 2.01f);
      gl.glTexCoord2f (ty2, tx1);
      gl.glVertex3f(2, 6, 2.01f);
      gl.glTexCoord2f(ty1,tx1);
      gl.glEnd();
      gl.glPopMatrix();

      paint3Texture.disable();
}

/**
* Draw the forth painting
**/
void drawPainting4(GL gl){

      // the process is the same as in the drawPainting1
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialWhite,0);

      TextureCoords tc = paint4Texture.getImageTexCoords ();
      float tx1 = tc.left ();
      float ty1 = tc.top ();
      float tx2 = tc.right ();
      float ty2 = tc.bottom ();
      
      paint4Texture.enable ();
      
      paint4Texture.bind ();

      gl.glPushMatrix();

      gl.glBegin(GL.GL_QUADS);
      gl.glVertex3f(6,7,16.99f);
      gl.glTexCoord2f (ty1, tx2);
      gl.glVertex3f(6,4, 16.99f);
      gl.glTexCoord2f (ty2, tx2);
      gl.glVertex3f(2, 4, 16.99f);
      gl.glTexCoord2f (ty2, tx1);
      gl.glVertex3f(2, 7, 16.99f);
      gl.glTexCoord2f(ty1,tx1);
      gl.glEnd();

      gl.glPopMatrix();

      paint4Texture.disable();
}

/**
* Draw the fifth painting
**/
void drawPainting5(GL gl){

     // the process is the same as in the drawPainting1
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialWhite,0);

      TextureCoords tc = paint5Texture.getImageTexCoords ();
      float tx1 = tc.left ();
      float ty1 = tc.top ();
      float tx2 = tc.right ();
      float ty2 = tc.bottom ();
      
      paint5Texture.enable ();
      
      paint5Texture.bind ();

      gl.glPushMatrix();

      gl.glBegin(GL.GL_QUADS);
      gl.glVertex3f(9.99f,6,12);
      gl.glTexCoord2f (ty1, tx2);
      gl.glVertex3f(9.99f,3, 12);
      gl.glTexCoord2f (ty2, tx2);
      gl.glVertex3f(9.99f, 3, 7);
      gl.glTexCoord2f (ty2, tx1);
      gl.glVertex3f(9.99f, 6, 7);
      gl.glTexCoord2f(ty1,tx1);
      gl.glEnd();

      gl.glPopMatrix();
      
      paint5Texture.disable();
}

/**
* Draw the floor and ceiling of the room,
* floor generated when flag is true and
* ceiling generated wnen flag is false
**/
void drawFloorCeiling(GL gl,int k,boolean flag){

      // set the material of the floor/ceiling
      float shine=0.25f;
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialWhite,0);
      gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialWhite,0);
      gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,shine*128.0f);

      //get the image coordinates and
      //save them to new variables
      TextureCoords tc = floorTexture.getImageTexCoords ();
      float tx1 = tc.left ();
      float ty1 = tc.top ();
      float tx2 = tc.right ();
      float ty2 = tc.bottom ();

      // choses whether to draw floor or ceiling
      if (flag == true){
         // enable texture for the floor
         floorTexture.enable ();

         // Bind this texture to the current rendering context.
         floorTexture.bind ();
      } else {
         //enable texture for thn ceiling
         ceilingTexture.enable ();

         // Bind this texture to the current rendering context.
         ceilingTexture.bind ();
      }

      gl.glPushMatrix();

      // draw 300 floor tails and
      // apply the floorTexture in each tile
      // or
      // draw the ceiling in 300 separate pieces and
      // apply the ceilingTexture in each piece
      for (int j=2;j<17;j++){
         for(int i=-10;i<10;i++){
            gl.glBegin(GL.GL_QUADS);
            gl.glVertex3i(i, k, j+1);
            gl.glTexCoord2f (ty1, tx2);
            gl.glVertex3i(i,k, j);
            gl.glTexCoord2f (ty2, tx2);
            gl.glVertex3i(i+1,k , j);
            gl.glTexCoord2f (ty2, tx1);
            gl.glVertex3i(i+1, k, j+1);
            gl.glTexCoord2f(ty1,tx1);
            gl.glEnd();   
         }
      }

      gl.glPopMatrix();

      // disable the proper texture
      if(flag == true)
            floorTexture.disable();
      else
          ceilingTexture.disable();
}

/**
* Draw the table
* consisting of a wooden table-top and
* four legs
**/
void drawTable(GL gl, GLUT glut){

     // draw the table-top
     // as a scaled wooden cube
     // using the wood list
     gl.glPushMatrix();
     gl.glTranslated(0, 2, 9);
     gl.glScaled(3, 0.1, 1.8);
     gl.glCallList(woodenCube);
     gl.glPopMatrix();

     // set the material of the four table-legs
     float shine=0.25f;
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialGreen,0);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialGreen,0);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialGreen,0);
     gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,shine*128.0f);

     // draw a cylinder as the first table-leg
     gl.glPushMatrix();
     gl.glTranslated(2,2,8);
     gl.glRotated(90, 1, 0, 0);
     glut.glutSolidCylinder(0.2, 3, 8, 8);

     // draw the second table-leg
     gl.glTranslated(-4,0,0);
      glut.glutSolidCylinder(0.2, 3, 8, 8);
     gl.glPopMatrix();

     // the third table-leg
     gl.glPushMatrix();
     gl.glTranslated(-2,2,10);
     gl.glRotated(90, 1, 0, 0);
      glut.glutSolidCylinder(0.2, 3, 8, 8);

     // the last table leg
     gl.glTranslated(4,0,0);
     glut.glutSolidCylinder(0.2, 3, 8, 8);
     gl.glPopMatrix();
     gl.glPopMatrix();
     
}

/**
* Draw the four chairs of the table
**/
void drawChairs(GL gl){

     // call the chair list four times,
     // draw the same chair in four different places

     gl.glPushMatrix();//right
     gl.glTranslated(-4, 1, 9);
     gl.glCallList(chair);
     gl.glPopMatrix();

     gl.glPushMatrix();//left
     gl.glTranslated(4, 1, 9);
     // rotate the chair in order to look towards the table
     gl.glRotated(180, 0, 1, 0);
     gl.glCallList(chair);
     gl.glPopMatrix();

     gl.glPushMatrix();//back
     gl.glTranslated(0, 1, 13);
     // rotate the chair in order to look towards the table
     gl.glRotated(90, 0, 1, 0);
     gl.glCallList(chair);
     gl.glPopMatrix();

     gl.glPushMatrix();//front
     gl.glTranslated(0, 1, 5);
     // rotate the chair in order to look towards the table
     gl.glRotated(270, 0, 1, 0);
     gl.glCallList(chair);
     gl.glPopMatrix();
}

/**
* Draw the light
**/
void drawLight(GLUT glut,GL gl){

     // draw the bulb as a white shpere,
     // first disable lighting because
     // the bulb emitis and not receive light
     gl.glDisable(GL.GL_LIGHTING);
     gl.glPushMatrix();
     gl.glTranslated(0, 8.65, 9);
     float shine=0.25f;
     gl.glColor3f(1, 1, 1);
     gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,shine*128.0f);
     glut.glutSolidSphere(0.25f, 100, 300);
     // enable lighting again to draw the rest of the light
     gl.glEnable(GL.GL_LIGHTING);

     // draw a black cube as the base of the bulb
     gl.glTranslated(0, 0.35, 0);
     gl.glScaled(1, 0.5, 1);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialBlack,0);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialBlack,0);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialBlack,0);
     glut.glutSolidCube(0.5f);

     gl.glPopMatrix();

}

/**
* Draw a wooden cube that is
* used to make the table-top and a part of each chair
**/
void WoodList(GL gl,GLUT glut){

     // generate the list named woodenCube
     woodenCube = gl.glGenLists(1);
     gl.glNewList(woodenCube,GL.GL_COMPILE);

     // set rthe material of the cube
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialWhite,0);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialWhite,0);
     gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialWhite,0);

     //get the image coordinates and
     //save them to new variables
     TextureCoords tc = woodTexture.getImageTexCoords ();
     float tx1 = tc.left ();
     float ty1 = tc.top ();
     float tx2 = tc.right ();
     float ty2 = tc.bottom ();
     
     gl.glBegin(GL.GL_QUADS);	// Start Drawing Quads

     // Top Face
        gl.glVertex3f(-1.0f,  1.0f, -1.0f);	// Top Left Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx2);
	gl.glVertex3f(-1.0f,  1.0f,  1.0f);	// Bottom Left Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx2);
	gl.glVertex3f( 1.0f,  1.0f,  1.0f);	// Bottom Right Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx1);
	gl.glVertex3f( 1.0f,  1.0f, -1.0f);	// Top Right Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx1);
     // Bottom Face
	gl.glVertex3f(-1.0f, -1.0f, -1.0f);	// Top Right Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx1);
	gl.glVertex3f( 1.0f, -1.0f, -1.0f);	// Top Left Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx2);
	gl.glVertex3f( 1.0f, -1.0f,  1.0f);	// Bottom Left Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx2);
	gl.glVertex3f(-1.0f, -1.0f,  1.0f);	// Bottom Right Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx1);
    // Front Face
	gl.glVertex3f(-1.0f, -1.0f,  1.0f);	// Bottom Left Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx2);
	gl.glVertex3f( 1.0f, -1.0f,  1.0f);	// Bottom Right Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx1);
	gl.glVertex3f( 1.0f,  1.0f,  1.0f);	// Top Right Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx1);
	gl.glVertex3f(-1.0f,  1.0f,  1.0f);	// Top Left Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx2);
    // Back Face
	gl.glVertex3f(-1.0f, -1.0f, -1.0f);	// Bottom Right Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx1);
	gl.glVertex3f(-1.0f,  1.0f, -1.0f);	// Top Right Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx1);
	gl.glVertex3f( 1.0f,  1.0f, -1.0f);	// Top Left Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx2);
	gl.glVertex3f( 1.0f, -1.0f, -1.0f);	// Bottom Left Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx2);
    // Right face
	gl.glVertex3f( 1.0f, -1.0f, -1.0f);	// Bottom Right Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx1);
	gl.glVertex3f( 1.0f,  1.0f, -1.0f);	// Top Right Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx1);
	gl.glVertex3f( 1.0f,  1.0f,  1.0f);	// Top Left Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx2);
	gl.glVertex3f( 1.0f, -1.0f,  1.0f);	// Bottom Left Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx2);
    // Left Face
	gl.glVertex3f(-1.0f, -1.0f, -1.0f);	// Bottom Left Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx2);
	gl.glVertex3f(-1.0f, -1.0f,  1.0f);	// Bottom Right Of The Texture and Quad
        gl.glTexCoord2f(ty2, tx1);
	gl.glVertex3f(-1.0f,  1.0f,  1.0f);	// Top Right Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx1);
	gl.glVertex3f(-1.0f,  1.0f, -1.0f);	// Top Left Of The Texture and Quad
        gl.glTexCoord2f(ty1, tx2);
    gl.glEnd();								// Done Drawing Quads

    // end code included in the list
    gl.glEndList();
}

/**
* Draw a chair consisting of
* two wooden parts and four legs
**/
void buildChairList(GL gl,GLUT glut){

    // generate the list named chair
    chair = gl.glGenLists(1);
    gl.glNewList(chair,GL.GL_COMPILE);

    // draw the wooden seat of the chair
    gl.glScaled(0.8, 0.1, 0.8);
    gl.glCallList(woodenCube);

    // draw the wooden back of the chair
    gl.glTranslated(-0.65, 15,0);
    gl.glScaled(0.1, 8, 1);
    gl.glCallList(woodenCube);

    // set the material of the four legs
    float shine=0.25f;
    gl.glMaterialfv(GL.GL_FRONT,GL.GL_AMBIENT,materialGreen,0);
    gl.glMaterialfv(GL.GL_FRONT,GL.GL_DIFFUSE,materialGreen,0);
    gl.glMaterialfv(GL.GL_FRONT,GL.GL_SPECULAR,materialGreen,0);
    gl.glMaterialf(GL.GL_FRONT,GL.GL_SHININESS,shine*128.0f);

    // draw each leg of the chair as a cylinder
    gl.glTranslated(-2,0,0.6);
    gl.glScaled(7, 1, 0.5);
    gl.glRotated(90, 1, 0, 0);
    glut.glutSolidCylinder(0.2, 4, 8, 8);

    gl.glTranslated(0,-2.5,0);
    glut.glutSolidCylinder(0.2, 4, 8, 8);

    gl.glTranslated(2,0,2);
    glut.glutSolidCylinder(0.2, 2, 8, 8);

    gl.glTranslated(0,2.5,0);
    glut.glutSolidCylinder(0.2, 2, 8, 8);

    // end code included in the list
    gl.glEndList();
}

/**
* Called when the GLDrawable (GLCanvas
* or GLJPanel) has changed in size.
* We don't need this
 **/
public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {}

/**
* If the display depth is changed while the
* program is running this method is called.
* We don't need this
**/
public void displayChanged(GLAutoDrawable glad, boolean bln, boolean bln1) {}

/**
 * Load an image from a file and retunn it as a texture
 **/
Texture load(String filename){
    Texture texture = null;

      try
      {
          // Create an OpenGL texture from the specified file. Do not create
          // mipmaps
          texture = TextureIO.newTexture (new File (filename), false);

          // Use the NEAREST magnification function when the pixel being
          // textured maps to an area less than or equal to one texture
          // element (texel)
          texture.setTexParameteri (GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST);

          // Use the NEAREST minification function when the pixel being
          // textured maps to an area greater than one texel
          texture.setTexParameteri (GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST);
      }
      catch (Exception e)
      {
          System.out.println ("error loading texture from "+filename);
      }

      return texture;
}

/**
 * Change the viewers aspect and rotate the scene
 * when the user types a number
 **/
public void keyTyped(KeyEvent e) {
      // the first four keys move the viewer into the x and z axis
      if (e.getKeyChar() == KeyEvent.VK_1)
          xPosition += 0.1;
      else if (e.getKeyChar() == KeyEvent.VK_2)
          xPosition -=0.1;
      else if (e.getKeyChar() == KeyEvent.VK_3)
          zPosition -=0.1;
      else if(e.getKeyChar() == KeyEvent.VK_4)
          zPosition +=0.1;
      // the last four keys rotate the scene around the y and x axis
      else if(e.getKeyChar() == KeyEvent.VK_5)
          angley -=1;
      else if(e.getKeyChar() == KeyEvent.VK_6)
          angley +=1;
      else if(e.getKeyChar() == KeyEvent.VK_7)
          anglex +=1;
      else if(e.getKeyChar() == KeyEvent.VK_8)
          anglex -=1;

      // repaint the GLCnvas so that the changes affect the scene
      glc.repaint();
    }

// We don't need the last two methods of KeyListener
public void keyPressed(KeyEvent e) {}

public void keyReleased(KeyEvent e) {}

}