import com.sun.opengl.util.GLUT;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureCoords;
import com.sun.opengl.util.texture.TextureIO;
import java.awt.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.GLU;
import java.io.*;

/**
* For our purposes only two of the
* GLEventListeners matter. Those would
* be init() and display().
*/
public class TextureView implements GLEventListener
{

    GLCanvas glc;
    private Texture wallTexture;

     void setGLCanvas(GLCanvas glcanvas) {
        this.glc = glcanvas;
    }
/**
* Take care of initialization here.
*/

 public void init(GLAutoDrawable gld) {

     wallTexture = load("Wand.jpg");
    }
/**
* Take care of drawing here.
*/
public void display(GLAutoDrawable drawable) {


            GL gl = drawable.getGL();
            GLU glu = new GLU();
            GLUT glut = new GLUT();
            gl.glMatrixMode(GL.GL_PROJECTION);
            gl.glLoadIdentity();
            // Aspect is width/height
            double w = ((Component) drawable).getWidth();
            double h = ((Component) drawable).getHeight();
            double aspect = w / h;
            //When using gluPerspective near and far need
            //to be positive.
            //The arguments are:
            //fovy, aspect, near, far
            glu.gluPerspective(60.0, aspect, 2.0, 10.0);
            gl.glMatrixMode(GL.GL_MODELVIEW);
            gl.glLoadIdentity();
            gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
            //bleach out the scene with too much light.
            //Define points for eye, at and up.
            //This is your camera. It ALWAYS goes
            //in the GL_MODELVIEW matrix.
            glu.gluLookAt(0, 0, 0, 0, 0, 20, 0, 1, 0);
            gl.glClear(GL.GL_COLOR_BUFFER_BIT);
            //Time to set up the light. We will only
            //use one light. If we used multiple lights,
            //we would want to dim them so as to not
            // bleach out the scene with too much light.

            float[] position = {0f, 15f, -30f, 0f};
            gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, position, 0);
            float[] diffuse = {.7f, .7f, .7f, 0f};
            gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse, 0);
            float[] ambient = {.2f, .2f, .2f, 0f};
            gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, ambient, 0);
            gl.glEnable(GL.GL_LIGHTING);
            gl.glEnable(GL.GL_LIGHT0);
            //GL_DEPTH_TEST prevents us from seeing polygons
            //that should be obscured. Remember to clear the
            //GL.GL_DEPTH_BUFFER_BIT
            gl.glEnable(GL.GL_DEPTH_TEST);
            //Notice the depth buffer bit is also being cleared
            gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

            //Placing icosahedron
            gl.glPushMatrix();
            gl.glTranslated(0, 0, 5);
            float[] material1 = {0.9f, 0.6f, 0.06f, 1.0f};
            gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE, material1, 0);
            glut.glutSolidIcosahedron();
            gl.glPopMatrix();
            //Placing cube
            gl.glPushMatrix();
            gl.glTranslated(3, 0, 5);
            float[] material2 = {0.9f, 0f, 0f, 1.0f};
            gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE, material2, 0);
            glut.glutSolidCube(1.2f);
            gl.glPopMatrix();
            //Placing sphere
            //The Cone parameters are:
            // radius of base
            // height of cone
            // slices or subdivisions around cone
            // stacks or subdivisions from base to peak
            gl.glPushMatrix();
            gl.glTranslated(-3, 0, 5);
            float[] material3 = {0.05f, 0f, 0.7f, 1.0f};
            gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE, material3, 0);
            glut.glutSolidSphere(0.8, 100, 360);
            gl.glPopMatrix();

             drawWall(gl,7,0,3,0);
             drawWall(gl,7,0,-3,0);
             drawWall(gl,-7,0,-3,0);
             drawWall(gl,-7,0,3,0);
    }
void drawWall(GL gl,int x1, int x2, int y1,int y2){
    
           float[] texMatMix = {1.0f, 1.0f, 1.0f, 1.0f};
           gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE, texMatMix, 0);
           TextureCoords tc = wallTexture.getImageTexCoords ();
           float tx1 = tc.left ();
           float ty1 = tc.top ();
           float tx2 = tc.right ();
           float ty2 = tc.bottom ();
           // Enable two-dimensional texturing.

           wallTexture.enable ();

           // Bind this texture to the current rendering context.

           wallTexture.bind ();

           //gl.glPushMatrix();
            
           gl.glBegin(GL.GL_QUADS);
           gl.glVertex3i(x2, y1, 8);
           gl.glTexCoord2f (ty1, tx2);
           gl.glVertex3i(x2, y2, 8);
           gl.glTexCoord2f (ty2, tx2);
           gl.glVertex3i(x1, y2, 8);
           gl.glTexCoord2f (ty2, tx1);
           gl.glVertex3i(x1, y1, 8);
           gl.glTexCoord2f(ty1,tx1);
           gl.glEnd();

           //gl.glPopMatrix();


           wallTexture.disable();
}

/**
* Called when the GLDrawable (GLCanvas
* or GLJPanel) has changed in size. We
* won't need this, but you may eventually
* need it -- just not yet.
*/
public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
    }

public void displayChanged(GLAutoDrawable glad, boolean bln, boolean bln1) {
    }

Texture load(String filename){
    Texture texture = null;

      try
      {
          // Create an OpenGL texture from the specified file. Do not create
          // mipmaps.

          texture = TextureIO.newTexture (new File (filename), false);

          // Use the NEAREST magnification function when the pixel being
          // textured maps to an area less than or equal to one texture
          // element (texel).

          texture.setTexParameteri (GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST);

          // Use the NEAREST minification function when the pixel being
          // textured maps to an area greater than one texel.

          texture.setTexParameteri (GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST);
      }
      catch (Exception e)
      {
          System.out.println ("error loading texture from "+filename);
      }

      return texture;
 }
}