import com.sun.opengl.util.GLUT;
import java.awt.*;
import java.awt.event.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.GLU;
/**
* For our purposes only two of the
* GLEventListeners matter. Those would
* be init() and display().
*/
public class TheTorusView implements GLEventListener, KeyListener
{

    double xPosition = 0;
    double zPosition = 0;
    double yPosition = 0;

    int theTorus;
    float anglex = 0;
    float angley = 0;

    float red = 0.0f;
    float green = 0.0f;
    float blue = 1.0f;


    GLCanvas glc;

     void setGLCanvas(GLCanvas glcanvas) {
        this.glc = glcanvas;
    }
/**
* Take care of initialization here.
*/

 
 public void init(GLAutoDrawable gld) {

    //The mode is GL.GL_MODELVIEW by default
    //We will also use the default ViewPort
    GLU glu = new GLU();
    GL gl = gld.getGL();

    gl.glMatrixMode(GL.GL_PROJECTION);
    gl.glLoadIdentity();

    // Aspect is width/height
    double w = ((Component) gld).getWidth();
    double h = ((Component) gld).getHeight();
    double aspect = w/h;
    //When using gluPerspective near and far need
    //to be positive.
    //The arguments are:
    //fovy, aspect, near, far
    glu.gluPerspective(60.0, aspect, 2.0, 20.0);

    theTorus = gl.glGenLists(1);
    gl.glNewList(theTorus, GL.GL_COMPILE);
    drawTorus(gl, 10, 25);
    gl.glEndList();
    }
/**
* Take care of drawing here.
*/
public void display(GLAutoDrawable drawable) {

    GL gl = drawable.getGL();
    GLU glu = new GLU();
    GLUT glut = new GLUT();

    gl.glMatrixMode(GL.GL_MODELVIEW);
    gl.glLoadIdentity();

    gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

    //Define points for eye, at and up.
    //This is your camera. It ALWAYS goes
    //in the GL_MODELVIEW matrix.
    glu.gluLookAt(
    xPosition, yPosition, zPosition,
    xPosition, yPosition, (zPosition+20),
    0, 1, 0
    );
    gl.glClear(GL.GL_COLOR_BUFFER_BIT);


    gl.glShadeModel(GL.GL_SMOOTH);
    gl.glEnable(GL.GL_LINE_SMOOTH);


    float position[] = {0f, 15f, -30f, 0f};
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, position,0);

    float diffuse[] = {.7f,.7f,.7f,0f};
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuse,0);

    float specular[] = {.2f,.2f,.2f,0f};
    gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, specular,0);

    float ambient[] = {.2f,.2f,.2f,0f};
    gl.glLightfv(GL.GL_LIGHT0,GL.GL_AMBIENT,ambient,3);

    gl.glEnable(GL.GL_LIGHTING);
    gl.glEnable(GL.GL_LIGHT0);
    //GL_DEPTH_TEST prevents us from seeing polygons
    //that should be obscured. Remember to clear the
    //GL.GL_DEPTH_BUFFER_BIT
    gl.glEnable(GL.GL_DEPTH_TEST);

    //Notice the depth buffer bit is also being cleared
    gl.glClear(GL.GL_COLOR_BUFFER_BIT|GL.GL_DEPTH_BUFFER_BIT);

    float material1[] = {red, green, blue, 1.0f};
    gl.glMaterialfv(GL.GL_FRONT_AND_BACK,GL.GL_AMBIENT_AND_DIFFUSE,material1,0);
    
    //transforming the place the next shape
    //will be drawn.
    gl.glTranslated(0, 0, 4);
    gl.glRotated(anglex, 1, 0, 0);
    gl.glRotated(angley, 0, 1, 0);

    //We use wire here because default
    //lighting is not good enough to
    //use when rendering the solid version
    gl.glCallList(theTorus);


    float material2[] = {1f, 0.2f, 0.2f, 1.0f};
    gl.glMaterialfv(GL.GL_FRONT_AND_BACK,GL.GL_AMBIENT_AND_DIFFUSE,material2,0);
    gl.glTranslated(0,0,2);
    gl.glCallList(theTorus);

    }

public void drawTorus(GL gl, int numc,int numt){
     double s, t, x, y, z, twopi = 2 * Math.PI;
    for (int i = 0; i < numc; i++)
    {
      gl.glBegin(GL.GL_QUAD_STRIP);
      for (int j = 0; j <= numt; j++)
      {
        for (int k = 1; k >= 0; k--)
        {
          s = (i + k) % numc + 0.5;
          t = j % numt;
          x = (1 + 0.1 * Math.cos(s * twopi / numc))
              * Math.cos(t * twopi / numt);
          y = (1 + 0.1 * Math.cos(s * twopi / numc))
              * Math.sin(t * twopi / numt);
          z = 0.1 * Math.sin(s * twopi / numc);
          gl.glVertex3d(x, y, z);
        }// k
      }// j
      gl.glEnd();
    }// i
 }
/**
* Called when the GLDrawable (GLCanvas
* or GLJPanel) has changed in size. We
* won't need this, but you may eventually
* need it -- just not yet.
*/
public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
    }
/**
* If the display depth is changed while the
* program is running this method is called.
* Nowadays this doesn't happen much, unless
* a programmer has his program do it.
*/
    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {
    }

    public void keyTyped(KeyEvent e) {
        if (e.getKeyChar() == KeyEvent.VK_1)
        zPosition += 1;
        else if (e.getKeyChar() == KeyEvent.VK_2)
        zPosition -= 1;
        else if (e.getKeyChar() == KeyEvent.VK_3)
        xPosition += 1;
        else if (e.getKeyChar() == KeyEvent.VK_4)
        xPosition -= 1;
        else if (e.getKeyChar() == KeyEvent.VK_5)
        yPosition += 1;
        else if (e.getKeyChar() == KeyEvent.VK_6)
        yPosition -= 1;
        else if (e.getKeyChar() == KeyEvent.VK_7)
        anglex += 1;
        else if (e.getKeyChar() == KeyEvent.VK_8)
        anglex -= 1;
        else if (e.getKeyChar() == KeyEvent.VK_9)
        angley += 1;
        else if (e.getKeyChar() == KeyEvent.VK_0)
        angley -= 1;
        glc.repaint();
    }

    public void keyPressed(KeyEvent ke) {

    }

    public void keyReleased(KeyEvent ke) {

    }

}